 /*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

void
xcave_create_TreeList_drinking(void)
{
	GtkListStore		*StoreList_drinking;
	GtkWidget			*scrolledwindow_drinking;
	GtkWidget			*TreeList_drinking;
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;
    XCAVE_DEBUG_B;

    scrolledwindow_drinking = lookup_widget(window_detail,"scrolledwindow_drinking");

    /* Creates a new tree store as with n_columns columns each of the types passed in. */
    StoreList_drinking = gtk_list_store_new(NUM_COLS_TREELIST_DRINKING, 
						GDK_TYPE_PIXBUF,
						G_TYPE_UINT, 
						G_TYPE_UINT, 
						G_TYPE_STRING, 
						G_TYPE_STRING,
						G_TYPE_UINT);

    /* create view TreeList_drinking */
    TreeList_drinking = gtk_tree_view_new_with_model (GTK_TREE_MODEL(StoreList_drinking));
    gtk_widget_show (TreeList_drinking);
    gtk_container_add (GTK_CONTAINER (scrolledwindow_drinking), TreeList_drinking);

    /* --- TreeList Column #5 -- DEGUSTATION SHEET --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("sheet"));
    renderer = gtk_cell_renderer_pixbuf_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "pixbuf", COL_TREELIST_ICON, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_drinking), col);

    /* --- TreeList Column #1 --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("year"));
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "text", COL_TREELIST_DRINKING_DATE, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_drinking), col);

    /* --- TreeList Column #2 -- QUANTITY --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("Qty"));
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "text", COL_TREELIST_DRINKING_QTY, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_drinking), col);

    /* --- TreeList Column #3 -- PRICE --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("Rated"));
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "text", COL_TREELIST_NOTE, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_drinking), col);

    /* --- TreeList Column #4 -- SUPPLIER --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("Comment"));
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "text", COL_TREELIST_COMMENTAIRE, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_drinking), col);

	/*
	 * 2.3.1 alternance de couleur de fond pour chaque ligne
	 */
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(TreeList_drinking),TRUE);

    g_signal_connect(G_OBJECT(TreeList_drinking), "row-activated", G_CALLBACK(on_TreeList_Drinking_double_click), NULL);
    GLADE_HOOKUP_OBJECT (window_detail, TreeList_drinking, "treelist_drinking");

    flag_drinking_doubleclick = FALSE;

    XCAVE_DEBUG_B;
    return;
}

GdkPixbuf  *
xcave_drinking_degustation_sheet_treatment(gint *tmpDegustationSheetId)
{
    GdkPixbuf     *icon;

    *tmpDegustationSheetId = 0;
    gchar          data[80];

    memset(data,'\0',80);

    if ( lDS != NULL) {
       if (lDS->degustation_sheet_id != (time_t)0) {
          degustation_sheet       *tmp_ds;

          tmp_ds = g_malloc ( sizeof(degustation_sheet) );
          memcpy(tmp_ds,lDS,sizeof(degustation_sheet));

          strcpy(data,"xcave_degustation.png");

          *tmpDegustationSheetId = (gint)tmp_ds->degustation_sheet_id;

          list_ds = g_list_append(list_ds, tmp_ds);
	  xcave_debug_msg_int(__FUNCTION__,"list_ds",2,g_list_length(list_ds));

      } 
      xcave_debug_msg(__FUNCTION__,"g_free(lDS)");
      g_free(lDS);
      lDS = NULL;
    }
    else {
         xcave_debug_msg(__FUNCTION__,"lDS null");
         strcpy(data,"");
    }

    icon = create_pixbuf(data);
    return(icon);
}

void
on_DrinkingOk_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget     *TreeList_drinking;
    GtkListStore  *StoreList_drinking_model;
    GtkTreeIter    pIter;
    Drinking      *tmpDrinking;
    GdkPixbuf     *icon;
    gint           tmpDegustationSheetId = 0;

    XCAVE_DEBUG_B;

    tmpDrinking = g_malloc (sizeof(Drinking));
    memset(tmpDrinking,'\0',sizeof(Drinking));

    tmpDrinking->Conso = atoi(
        gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_year"))));
    tmpDrinking->qty = atoi(
        gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "spinbutton_drinking_qty"))));
    strcpy(tmpDrinking->Note,
        gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_rated"))));
    strcpy(tmpDrinking->Commentaire,
        gtk_entry_get_text (GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_comment"))));

    if ( flag_drinking_doubleclick == FALSE ) {
    /*
     * degustation sheet
     */
    icon = xcave_drinking_degustation_sheet_treatment(&tmpDegustationSheetId);

    TreeList_drinking = lookup_widget(window_detail,"treelist_drinking");
    StoreList_drinking_model	= (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)TreeList_drinking);

    gtk_list_store_append(StoreList_drinking_model, &pIter);
    gtk_list_store_set(StoreList_drinking_model, &pIter,
                        COL_TREELIST_ICON,icon,
                        COL_TREELIST_DRINKING_DATE, (guint)tmpDrinking->Conso,
                        COL_TREELIST_DRINKING_QTY, (guint)tmpDrinking->qty,
                        COL_TREELIST_NOTE, tmpDrinking->Note,
                        COL_TREELIST_COMMENTAIRE, tmpDrinking->Commentaire,
                        COL_TREELIST_SHEET_ID,tmpDegustationSheetId,
                        -1);			

    NbBottleDetail -= tmpDrinking->qty;
    NbBottleStock  -= tmpDrinking->qty;

    gchar   ach[purchaseDate_size];
    sprintf (ach, "%d", nowan);
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(window_detail,"entry_drinking_year")),(gchar *)ach);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_drinking_qty")), (guint) 1);
    gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_rated")),"");
    gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_comment")),"");

    /*
     * update de la statusbar indiquant le nb de bouteille du vin selectionne
     * se trouvant dans le stock
     */
    xcave_detail_statusbar_v = lookup_widget (window_detail, "xcave_detail_statusbar");
    sprintf (xcave_detail_statusbar_txt, _("Number of bottles for the selection : %d"), NbBottleDetail);
    if (xcave_detail_statusbar_id != 0) gtk_statusbar_remove ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_id);
    xcave_detail_statusbar_id = gtk_statusbar_push ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_txt);

    } else {

    GtkTreeSelection  *selection;
    gint               tmpQty;
    GdkPixbuf         *icon;

    /*
     * degustation sheet
     */
    icon = xcave_drinking_degustation_sheet_treatment(&tmpDegustationSheetId);

    TreeList_drinking = lookup_widget(window_detail,"treelist_drinking");
    StoreList_drinking_model	= (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)TreeList_drinking);

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(TreeList_drinking));
    /*
     *  on test si une ligne est sélectionnée
     */
    if(gtk_tree_selection_get_selected(selection, (GtkTreeModel **)&StoreList_drinking_model, &pIter)){
      gtk_tree_model_get ((GtkTreeModel *)StoreList_drinking_model, &pIter, COL_TREELIST_DRINKING_QTY, &tmpQty, -1);
       NbBottleDetail += tmpQty;
       NbBottleStock  += tmpQty;
       gtk_list_store_set(StoreList_drinking_model, &pIter,
                            COL_TREELIST_ICON,icon,
                            COL_TREELIST_DRINKING_DATE, (guint)tmpDrinking->Conso,
                            COL_TREELIST_DRINKING_QTY, (guint)tmpDrinking->qty,
                            COL_TREELIST_NOTE, tmpDrinking->Note,
                            COL_TREELIST_COMMENTAIRE, tmpDrinking->Commentaire,
                            COL_TREELIST_SHEET_ID, tmpDegustationSheetId,
                            -1);			

       NbBottleDetail -= tmpDrinking->qty;
       NbBottleStock  -= tmpDrinking->qty;

       gchar          ach[purchaseDate_size];
       sprintf (ach, "%d", nowan);
       gtk_entry_set_text(GTK_ENTRY(lookup_widget(window_detail,"entry_drinking_year")),(gchar *)ach);
       gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_drinking_qty")), (guint) 1);
       gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_rated")),"");
       gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_comment")),"");

       /*
       * update de la statusbar indiquant le nb de bouteille du vin selectionne
       * se trouvant dans le stock
       */
       xcave_detail_statusbar_v = lookup_widget (window_detail, "xcave_detail_statusbar");
       sprintf (xcave_detail_statusbar_txt, _("Number of bottles for the selection : %d"), NbBottleDetail);
       if (xcave_detail_statusbar_id != 0) gtk_statusbar_remove ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_id);
          xcave_detail_statusbar_id = gtk_statusbar_push ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_txt);
       }
       else {
         GtkWidget	*dialog;
         dialog = gtk_message_dialog_new ((GtkWindow *)window_detail,
                                GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_ERROR,
                                  GTK_BUTTONS_CLOSE,
                                  _("You must select a line!!!")
                                  );
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
	}
	
    }

    flag_drinking_doubleclick = FALSE;
    g_free(tmpDrinking);

    XCAVE_DEBUG_B;
}

gboolean
on_TreeList_Drinking_double_click (GtkWidget       *widget,
                                   GdkEventButton  *event,
                                   gpointer         user_data)
{
    gchar             *tmpNote;
    gchar             *tmpComment;	
    GtkWidget         *TreeList_drinking;
    GtkListStore      *StoreList_drinking_model;
    GtkTreeIter         pIter;
    Drinking          *tmpDrinking;
    GtkTreeSelection  *selection;
	
    XCAVE_DEBUG_B;

    tmpDrinking = g_malloc (sizeof(Drinking));

    memset(tmpDrinking,'\0',sizeof(Drinking));
    TreeList_drinking = lookup_widget(window_detail,"treelist_drinking");
    StoreList_drinking_model = (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)TreeList_drinking);
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(TreeList_drinking));

    gtk_tree_selection_get_selected(selection, (GtkTreeModel **)&StoreList_drinking_model, (GtkTreeIter *)&pIter);
    gtk_tree_model_get ((GtkTreeModel *)StoreList_drinking_model, &pIter,
                         COL_TREELIST_DRINKING_DATE, &tmpDrinking->Conso,
                         COL_TREELIST_DRINKING_QTY, &tmpDrinking->qty,
                         COL_TREELIST_NOTE, &tmpNote,
                         COL_TREELIST_COMMENTAIRE, &tmpComment,
                         COL_TREELIST_SHEET_ID, &tmpDrinking->degustation_sheet_id,
                         -1);

    gchar          ach[purchaseDate_size];
    sprintf (ach, "%d", tmpDrinking->Conso);
    gtk_entry_set_text(GTK_ENTRY(lookup_widget(window_detail,"entry_drinking_year")),(gchar *)ach);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_drinking_qty")), (guint)tmpDrinking->qty);
    gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_rated")),tmpNote);
    gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_drinking_comment")),tmpComment);

    if ((gint)tmpDrinking->degustation_sheet_id != 0){
        /*
         * degustation sheet id found
         * save the id, so when degustation_windows will be activate, I can found again
         * the degustation sheet data 
         */
	xcave_debug_msg(__FUNCTION__,"g_malloc(lDS)");
	lDS = g_malloc ( sizeof(degustation_sheet) );
	memset(lDS, '\0', sizeof( degustation_sheet ));
	xcave_debug_ptr_int(__FUNCTION__,(gchar *)lDS,0);
        lDS->degustation_sheet_id = (time_t)tmpDrinking->degustation_sheet_id;
        xcave_debug_2str(__FUNCTION__,"degustation_sheet_d",25,ctime(&lDS->degustation_sheet_id));
    }
    g_free(tmpDrinking);
    g_free(tmpNote);
    g_free(tmpComment);

    flag_drinking_doubleclick = TRUE;

    XCAVE_DEBUG_B;

    return FALSE;
}

void
on_DrinkingDelete_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget        *TreeList_drinking;
    GtkListStore     *StoreList_drinking_model;
    GtkTreeIter       pIter;
    GtkTreeSelection *selection;
    gint              tmpQty;
    gint              tmpDS = 0;

    XCAVE_DEBUG_B;

    TreeList_drinking = lookup_widget(window_detail,"treelist_drinking");
    StoreList_drinking_model = (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)TreeList_drinking);
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(TreeList_drinking));

    if(gtk_tree_selection_get_selected (selection, (GtkTreeModel **) &StoreList_drinking_model, &pIter)){
        /*
         *  2.2.9 qty false after removing a drinking wine
         */
        gtk_tree_model_get ((GtkTreeModel *)StoreList_drinking_model, &pIter, 
                                COL_TREELIST_DRINKING_QTY, &tmpQty,
                                COL_TREELIST_SHEET_ID, &tmpDS, -1);
        NbBottleDetail += tmpQty;
        NbBottleStock  += tmpQty;
        /*
         * remove associated degustation sheet
         */
	if (tmpDS != 0) {
           degustation_sheet    *tmp_ds;
           GList                *tmp_list_ds; 

	   tmp_ds = g_malloc ( sizeof(degustation_sheet) );
           /*
            * controle et suppression de la liste temporaire (list_ds) celle utilisee lors de l ajout de 
            * ligne drinking pour un meme vin
            */
           for(tmp_list_ds = list_ds; tmp_list_ds != NULL ; tmp_list_ds = tmp_list_ds->next) {
              memcpy(tmp_ds,tmp_list_ds->data,sizeof(degustation_sheet));
              if (tmpDS == tmp_ds->degustation_sheet_id) {
                 list_ds = g_list_remove_link(list_ds, tmp_list_ds);
                 g_list_free_1(tmp_list_ds);
                 break; 
	      }
	   }
           /*
            * controle et suppression de la liste degustation (list_full_ds) qui contient l ensemble des
            * fiches de degustations de la cave
            */
           for(tmp_list_ds = list_full_ds; tmp_list_ds != NULL ; tmp_list_ds = tmp_list_ds->next) {
              memcpy(tmp_ds,tmp_list_ds->data,sizeof(degustation_sheet));
              if (tmpDS == tmp_ds->degustation_sheet_id) {
                 list_full_ds = g_list_remove_link(list_full_ds, tmp_list_ds);
                 g_list_free_1(tmp_list_ds);
                 break; 
	      }
	   }
	}

        /*
         * remove the drinking line
         */
        gtk_list_store_remove( StoreList_drinking_model, &pIter);
        xcave_detail_statusbar_v = lookup_widget (window_detail, "xcave_detail_statusbar");
        sprintf (xcave_detail_statusbar_txt, _("Number of bottles for the selection : %d"), NbBottleDetail);
        if (xcave_detail_statusbar_id != 0) gtk_statusbar_remove ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_id);
        xcave_detail_statusbar_id = gtk_statusbar_push ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_txt);
    }
    else {
         GtkWidget	*dialog;
         dialog = gtk_message_dialog_new ((GtkWindow *)window_detail,
                               GTK_DIALOG_DESTROY_WITH_PARENT,
                               GTK_MESSAGE_ERROR,
                               GTK_BUTTONS_CLOSE,
                               _("You must select a line!!!")
                               );
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);
    }

    flag_drinking_doubleclick = FALSE;

    XCAVE_DEBUG_B;

}


