/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

void
xcave_create_TreeList_purchase(void)
{
    GtkListStore	*StoreList_purchase;
    GtkWidget		*xcave_detail_purchase_scrolledwindow;
    GtkWidget		*TreeList_purchase;
    GtkTreeViewColumn   *col;
    GtkCellRenderer     *renderer;

    XCAVE_DEBUG_B;

    xcave_detail_purchase_scrolledwindow = lookup_widget(window_detail,"xcave_detail_purchase_scrolledwindow");

    /* Creates a new tree store as with n_columns columns each of the types passed in. */
    StoreList_purchase = gtk_list_store_new(NUM_COLS_TREELIST_PURCHASE, 
					G_TYPE_UINT, 
					G_TYPE_UINT, 
					G_TYPE_STRING, 
					G_TYPE_STRING, 
					G_TYPE_STRING);

    /* create view TreeList_purchase */
    TreeList_purchase = gtk_tree_view_new_with_model (GTK_TREE_MODEL(StoreList_purchase));
    gtk_widget_show (TreeList_purchase);
    gtk_container_add (GTK_CONTAINER (xcave_detail_purchase_scrolledwindow), TreeList_purchase);

    /* --- TreeList Column #1 --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("year"));
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "text", COL_TREELIST_PURCHASE_DATE, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);

    /* --- TreeList Column #2 -- QUANTITY --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("Qty"));
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "text", COL_TREELIST_PURCHASE_QTY, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);

    /* --- TreeList Column #3 -- PRICE --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("Price"));
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "text", COL_TREELIST_PURCHASE_PRICE, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);

    /* --- TreeList Column #4 -- SUPPLIER --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("Supplier"));
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "text", COL_TREELIST_SUPPLIER, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);

    /* --- TreeList Column #5 -- SUPPLIER ADDRESS --- */
    col = gtk_tree_view_column_new();
    gtk_tree_view_column_set_title(col, _("Supplier address"));
    renderer = gtk_cell_renderer_text_new();
    gtk_tree_view_column_pack_start(col, renderer, TRUE);
    gtk_tree_view_column_set_attributes(col, renderer, "text", COL_TREELIST_SUPPLIER_ADR, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW(TreeList_purchase), col);
	
	/*
	 * 2.3.1 alternance de couleur de fond pour chaque ligne
	 */
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(TreeList_purchase),TRUE);

    g_signal_connect(G_OBJECT(TreeList_purchase), "row-activated", G_CALLBACK(on_TreeList_Purchase_double_click), NULL);
	
    GLADE_HOOKUP_OBJECT (window_detail, TreeList_purchase, "treelist_purchase");

    flag_purchase_doubleclick = FALSE;

    XCAVE_DEBUG_E;
    return;
}

void
on_PurchaseOk_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{

    GtkWidget	   *TreeList_purchase;
    GtkListStore   *StoreList_purchase_model;
    GtkTreeIter     pIter;
    Purchase	   *tmpPurchase;

    XCAVE_DEBUG_B;

    tmpPurchase = g_malloc (sizeof(Purchase));
    memset(tmpPurchase,'\0',sizeof(Purchase));

    strcpy(tmpPurchase->Supplier,
           gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "entry_purchase_supplier"))));
    strcpy(tmpPurchase->SupplierAdr,
           gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "entry_purchase_supplier_adr"))));
    tmpPurchase->Achat = atoi(
	       gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "entry_purchase_year"))));

    tmpPurchase->qty = atoi(
           gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "spinbutton_purchase_qty"))));

	/*
	 * 2.3.1 Pb nbr de decimal apres la virgule, dans le cas de l'utilisation
	 *       d'un float
	 */
	gchar tmp_price[12];
	g_snprintf(tmp_price,sizeof(tmp_price),"%.2f",atof(
           gtk_entry_get_text (GTK_ENTRY (lookup_widget(window_detail, "spinbutton_purchase_price")))));

    TreeList_purchase = lookup_widget(window_detail,"treelist_purchase");
    StoreList_purchase_model = (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)TreeList_purchase);

    if (flag_purchase_doubleclick == FALSE) {
        gtk_list_store_append(StoreList_purchase_model, &pIter);
        gtk_list_store_set(StoreList_purchase_model, &pIter,
                                                 COL_TREELIST_PURCHASE_DATE, (guint)tmpPurchase->Achat,
                                                 COL_TREELIST_PURCHASE_QTY, (guint)tmpPurchase->qty,
                                                 COL_TREELIST_PURCHASE_PRICE, &tmp_price,
                                                 COL_TREELIST_SUPPLIER, &tmpPurchase->Supplier,
                                                 COL_TREELIST_SUPPLIER_ADR, &tmpPurchase->SupplierAdr,
                                                 -1);			
    
        NbBottleDetail += tmpPurchase->qty;
        NbBottleStock  += tmpPurchase->qty;

        gchar          ach[purchaseDate_size];
        sprintf (ach, "%d", nowan);
        gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_year")),(gchar *) ach);
		guint digits = 2;
		gtk_spin_button_set_digits(GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_price")), digits);
        gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_price")), (gfloat) 0);
        gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_qty")), (guint) 1);
        /*
         *     2.2.6
         *
         */
        gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_supplier"))," ");
        gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_supplier_adr")),"");
	
        /*
        * update de la statusbar indiquant le nb de bouteille du vin selectionne
        * se trouvant dans le stock
        */
        xcave_detail_statusbar_v = lookup_widget (window_detail, "xcave_detail_statusbar");
        sprintf (xcave_detail_statusbar_txt, _("Number of bottles for the selection : %d"), NbBottleDetail);
        if (xcave_detail_statusbar_id != 0) gtk_statusbar_remove ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_id);
        xcave_detail_statusbar_id = gtk_statusbar_push ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_txt);
	
    } else {

        GtkTreeSelection *selection;
        gint tmpQty;

	    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(TreeList_purchase));

	    if(gtk_tree_selection_get_selected (selection,
                                            (GtkTreeModel **)&StoreList_purchase_model,
                                            &pIter)){
		  gtk_tree_model_get ((GtkTreeModel *)StoreList_purchase_model, &pIter,
												COL_TREELIST_PURCHASE_QTY, &tmpQty, -1);
		  NbBottleDetail -= tmpQty;
		  NbBottleStock  -= tmpQty;
		  gtk_list_store_set(StoreList_purchase_model, &pIter,
							COL_TREELIST_PURCHASE_DATE, (guint)tmpPurchase->Achat,
							COL_TREELIST_PURCHASE_QTY, (guint)tmpPurchase->qty,
							COL_TREELIST_PURCHASE_PRICE, &tmp_price,
							COL_TREELIST_SUPPLIER, &tmpPurchase->Supplier,
							COL_TREELIST_SUPPLIER_ADR, &tmpPurchase->SupplierAdr,
									-1);			

		  NbBottleDetail += tmpPurchase->qty;
		  NbBottleStock  += tmpPurchase->qty;

		  gchar          ach[purchaseDate_size];
		  sprintf (ach, "%d", nowan);
		  gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_year")),(gchar *) ach);
		  guint digits = 2;
		  gtk_spin_button_set_digits(GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_price")), digits);
		  gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_price")), (gfloat) 0);
		  gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_qty")), (guint) 1);
		  /*
		   * 2.2.6
		   *
		   */
		  gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_supplier"))," ");
		  gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_supplier_adr")),"");
		
		  /*
		   * update de la statusbar indiquant le nb de bouteille du vin selectionne
		   * se trouvant dans le stock
		   */
		  xcave_detail_statusbar_v = lookup_widget (window_detail, "xcave_detail_statusbar");
		  sprintf (xcave_detail_statusbar_txt, _("Number of bottles for the selection : %d"), NbBottleDetail);
		  if (xcave_detail_statusbar_id != 0) gtk_statusbar_remove ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_id);
		  xcave_detail_statusbar_id = gtk_statusbar_push ((GtkStatusbar *)xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_txt);
          }
          else {
                quick_message (_("You must select a line!!!"));
        }
    }

    g_free(tmpPurchase);

    flag_purchase_doubleclick = FALSE;

    XCAVE_DEBUG_E;
}

gboolean
on_TreeList_Purchase_double_click (GtkWidget       *widget,
                                   GdkEventButton  *event,
                                   gpointer         user_data)
{
    GtkWidget        *TreeList_purchase;
    GtkListStore     *StoreList_purchase_model;
    GtkTreeIter       Iter;
    GtkTreeSelection *selection;

    gint             date_achat = 0;
    gint             qty = 0;
    gfloat           price = 0.0;
    gchar            *pSupplier = NULL;
    gchar            *pSupplierAdr = NULL;	
	gchar            *pPrice = NULL;
	
    XCAVE_DEBUG_B;

    TreeList_purchase = lookup_widget(window_detail,"treelist_purchase");
    StoreList_purchase_model = (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)TreeList_purchase);
    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(TreeList_purchase));

    gtk_tree_selection_get_selected(selection,
                                    (GtkTreeModel **)&StoreList_purchase_model,
                                    (GtkTreeIter *)&Iter);

	/* 2.4 - price always zero on double click */
    gtk_tree_model_get ((GtkTreeModel *)StoreList_purchase_model, &Iter,
                                                                  COL_TREELIST_PURCHASE_DATE, &date_achat,
                                                                  COL_TREELIST_PURCHASE_QTY, &qty,
                                                                  COL_TREELIST_PURCHASE_PRICE, &pPrice,
                                                                  COL_TREELIST_SUPPLIER, (gchar *)&pSupplier,
                                                                  COL_TREELIST_SUPPLIER_ADR,(gchar *)&pSupplierAdr,
                                                                  -1);

    gchar ach[purchaseDate_size + 1];
    sprintf (ach, "%4d", date_achat);
    gtk_entry_set_text (GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_year")),(gchar *) ach);

	price = atof(pPrice);
	guint digits = 2;
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_price")), digits);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_price")), (gfloat) price);
    gtk_spin_button_set_value (GTK_SPIN_BUTTON (lookup_widget (window_detail, "spinbutton_purchase_qty")), (guint) qty);
    gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_supplier_adr")),(gchar *)pSupplierAdr);
    gtk_entry_set_text(GTK_ENTRY (lookup_widget (window_detail, "entry_purchase_supplier")),(gchar *)pSupplier);

    flag_purchase_doubleclick = TRUE;

    XCAVE_DEBUG_E;
	
    return FALSE;

}
void
on_PurchaseDelete_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget	*TreeList_purchase;
	GtkListStore	*StoreList_purchase_model;
	GtkTreeIter 	pIter;
	GtkTreeSelection *selection;
	gint tmpQty;
	gchar		xcave_detail_statusbar_txt[80];
	GtkStatusbar	*xcave_detail_statusbar_v;
	
    XCAVE_DEBUG_B;

	TreeList_purchase = lookup_widget(window_detail,"treelist_purchase");
	StoreList_purchase_model = (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)TreeList_purchase);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(TreeList_purchase));

	if(gtk_tree_selection_get_selected     (selection,
                                       (GtkTreeModel **)&StoreList_purchase_model,
                                      &pIter)){
																				
		gtk_tree_model_get ((GtkTreeModel *)StoreList_purchase_model, &pIter, COL_TREELIST_PURCHASE_QTY, &tmpQty, -1);
		NbBottleDetail -= tmpQty;
		NbBottleStock  -= tmpQty;
		gtk_list_store_remove( StoreList_purchase_model,
													&pIter);
		xcave_detail_statusbar_v = (GtkStatusbar *) lookup_widget (window_detail, "xcave_detail_statusbar");
		sprintf (xcave_detail_statusbar_txt, _("Number of bottles for the selection : %d"), NbBottleDetail);
		if (xcave_detail_statusbar_id != 0) gtk_statusbar_remove (xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_id);
		xcave_detail_statusbar_id = gtk_statusbar_push (xcave_detail_statusbar_v, xcave_detail_statusbar_ctxtid, xcave_detail_statusbar_txt);
	}
	else {
		 GtkWidget	*dialog;
		 dialog = gtk_message_dialog_new ((GtkWindow *)window_detail,
                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_ERROR,
                                  GTK_BUTTONS_CLOSE,
                                  _("You must select a line!!!")
                                  );
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
	}

    flag_purchase_doubleclick = FALSE;

    XCAVE_DEBUG_E;
}

