/*
 * Copyright (C) 2003 Jean-Philippe Hergott <hergott@free.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
/*
 * supp_modif clicked -> display
 * GtkWindow "ConSupp".
 */

void
on_xcave_button_supp_clicked   (GtkButton       *button,
                                       gpointer         user_data)
{
	Vin		*Vin_rowSelected;
    XCAVE_DEBUG_B;

	/*
	 * we need selected data 
	 */
	Vin_rowSelected = xcave_id_selected_treelist_row();

	if (Vin_rowSelected != NULL) {

		/* 
		 * desactive window_xcave
		 */
		gtk_widget_set_sensitive(window_xcave, FALSE);
	
		confirm = create_xcave_supp ();
		gtk_widget_show (confirm);
	}
    XCAVE_DEBUG_E;

}

/*
 * GtkWindow "ConSupp".
 * cancel_button clicked
 */
void
on_xcave_supp_button_cancel_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_object_destroy (GTK_OBJECT (confirm));

	/* 
	 * reactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, TRUE);
}

/*
 * GtkWindow "ConSupp".
 * cancel_button clicked
 */
void
on_xcave_supp_button_ok_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkTreeSelection *selection;
	GtkTreeModel     *model;
	GtkTreeIter       selected_row;
	GtkWidget		 *TreeList;

	GList  *list_stock_rowSelected;
	ptrVin	Vin_rowSelected;

    XCAVE_DEBUG_B;

	TreeList = lookup_widget(window_xcave,"treeview_xcave_wine_information");
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(TreeList));
	g_assert(gtk_tree_selection_get_mode(selection) == GTK_SELECTION_SINGLE);

    if (gtk_tree_selection_get_selected(selection, &model, &selected_row)) {
       /*
        * update list_stock by removing selected wine
        */
        gtk_tree_model_get (model, &selected_row, 
                            COL_TREELIST_PDATA, &Vin_rowSelected,
                            COL_TREELIST_PGLIST, &list_stock_rowSelected,
                            -1);
       /*
        * suppression des fiches de degustations associees
        */

        list_stock = g_list_remove_link(list_stock,list_stock_rowSelected);
        list_stock = g_list_sort(list_stock,(GCompareFunc)cmp_stock);
        xcave_xmlWrite(DBPATH);

       /*
        * update TreeList
        */
       /* 2.3.1 gtk_list_store_remove(GTK_TREE_VIEW(model), &selected_row); */
        gtk_tree_store_remove(GTK_TREE_STORE(model), &selected_row);
	}

	/*
	 * need to update storeview_xcave_wine_origin and storelist
	 */
	GtkWidget		*treelist;
	GtkWidget		*treeview;
	// 2.3.1. GtkListStore	*storeview_xcave_wine_information;
	GtkTreeStore	*storeview_xcave_wine_information;
	GtkTreeStore	*storeview_xcave_wine_origin;
	
	treelist = lookup_widget(window_xcave,"treeview_xcave_wine_information");
	// 2.3.1 storeview_xcave_wine_information = (GtkListStore *) gtk_tree_view_get_model((GtkTreeView *)treelist);
	storeview_xcave_wine_information = (GtkTreeStore *) gtk_tree_view_get_model((GtkTreeView *)treelist);
	// 2.3.1 gtk_list_store_clear(storeview_xcave_wine_information);
	gtk_tree_store_clear(storeview_xcave_wine_information);

	treeview = lookup_widget(window_xcave,"treeview_xcave_wine_origin");
	storeview_xcave_wine_origin	= (GtkTreeStore *) gtk_tree_view_get_model((GtkTreeView *)treeview);
	gtk_tree_store_clear(storeview_xcave_wine_origin);
	
	xcave_fill_model (storeview_xcave_wine_origin, storeview_xcave_wine_information);

	/*
	 * penser à mettre à jour la liste emplacement
	 */
	list_append_rack();

	/*
	 *
	 */
	NbBottleStock -= Vin_rowSelected->qty;
    if (xcave_statusbar_id != 0) 
       gtk_statusbar_remove ((GtkStatusbar *)xcave_statusbar_v, xcave_statusbar_ctxtid, xcave_statusbar_id);

    sprintf (xcave_statusbar_txt, _("Number of bottles in cellar : %d"), NbBottleStock);
    xcave_statusbar_id = gtk_statusbar_push ((GtkStatusbar *)xcave_statusbar_v, xcave_statusbar_ctxtid, xcave_statusbar_txt);
	
	
	gtk_object_destroy (GTK_OBJECT (confirm));
	
	/* 
	 * reactive window_xcave
	 */
	gtk_widget_set_sensitive(window_xcave, TRUE);
	
    XCAVE_DEBUG_E;
}

void
on_xcave_supp_destroy   (GtkObject       *object,
                          gpointer         user_data)
{

   gtk_object_destroy (GTK_OBJECT (confirm));

   /* 
    * reactive window_xcave
    */
   gtk_widget_set_sensitive(window_xcave, TRUE);
}
